<?php

namespace Drupal\mollie_payment\Plugin\Payment\Method;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\Core\Utility\Token;
use Drupal\payment\Entity\Payment;
use Drupal\payment\Entity\PaymentInterface;
use Drupal\payment\EventDispatcherInterface;
use Drupal\payment\Plugin\Payment\Status\PaymentStatusManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\payment\OperationResult;
use Drupal\payment\Plugin\Payment\Method\PaymentMethodBase;
use Drupal\payment\Plugin\Payment\Method\PaymentMethodCapturePaymentInterface;
use Drupal\payment\Plugin\Payment\Method\PaymentMethodRefundPaymentInterface;
use Drupal\payment\Plugin\Payment\Method\PaymentMethodUpdatePaymentStatusInterface;
use Drupal\payment\Response\Response;
use Drupal\mollie_payment\Entity\MollieProfile;
use Mollie_API_Client;

/**
 * A payment method using Mollie.
 *
 * Plugins that extend this class must have the following keys in their plugin
 * definitions:
 * - entity_id: (string) The ID of the payment method entity the plugin is for.
 * - execute_status_id: (string) The ID of the payment status plugin to set at
 *   payment execution.
 * - capture: (boolean) Whether or not payment capture is supported.
 * - capture_status_id: (string) The ID of the payment status plugin to set at
 *   payment capture.
 * - refund: (boolean) Whether or not payment refunds are supported.
 * - refund_status_id: (string) The ID of the payment status plugin to set at
 *   payment refund.
 *
 * @PaymentMethod(
 *   deriver = "Drupal\mollie_payment\Plugin\Payment\Method\MolliePaymentDeriver",
 *   id = "mollie_payment",
 *   operations_provider = "\Drupal\mollie_payment\Plugin\Payment\Method\MolliePaymentOperationsProvider",
 * )
 */
class MolliePayment extends PaymentMethodBase implements ContainerFactoryPluginInterface, PaymentMethodCapturePaymentInterface, PaymentMethodRefundPaymentInterface, PaymentMethodUpdatePaymentStatusInterface {

  /**
   * The payment status manager.
   *
   * @var \Drupal\payment\Plugin\Payment\Status\PaymentStatusManagerInterface
   */
  protected $paymentStatusManager;

  /**
   * Constructs a new instance.
   *
   * @param mixed[] $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed[] $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\payment\EventDispatcherInterface $event_dispatcher
   *   The event dispatcher.
   * @param \Drupal\Core\Utility\Token $token
   *   The token API.
   * @param \Drupal\payment\Plugin\Payment\Status\PaymentStatusManagerInterface
   *   The payment status manager.
   */
  public function __construct(array $configuration, $plugin_id, array $plugin_definition, ModuleHandlerInterface $module_handler, EventDispatcherInterface $event_dispatcher, Token $token, PaymentStatusManagerInterface $payment_status_manager) {
    $configuration += $this->defaultConfiguration();
    parent::__construct($configuration, $plugin_id, $plugin_definition, $module_handler, $event_dispatcher, $token, $payment_status_manager);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static($configuration, $plugin_id, $plugin_definition, $container->get('module_handler'), $container->get('payment.event_dispatcher'), $container->get('token'), $container->get('plugin.manager.payment.status'));
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedCurrencies() {
    return TRUE;
  }

  /**
   * Gets the ID of the payment method this plugin is for.
   *
   * @return string
   */
  public function getEntityId() {
    return $this->pluginDefinition['entity_id'];
  }

  /**
   * Gets the status to set on payment execution.
   *
   * @return string
   *   The plugin ID of the payment status to set.
   */
  public function getExecuteStatusId() {
    return $this->pluginDefinition['execute_status_id'];
  }

  /**
   * Gets the status to set on payment capture.
   *
   * @return string
   *   The plugin ID of the payment status to set.
   */
  public function getCaptureStatusId() {
    return $this->pluginDefinition['capture_status_id'];
  }

  /**
   * Gets whether or not capture is supported.
   *
   * @param bool
   *   Whether or not to support capture.
   */
  public function getCapture() {
    return $this->pluginDefinition['capture'];
  }

  /**
   * Gets the status to set on payment refund.
   *
   * @return string
   *   The plugin ID of the payment status to set.
   */
  public function getRefundStatusId() {
    return $this->pluginDefinition['refund_status_id'];
  }

  /**
   * Gets whether or not capture is supported.
   *
   * @param bool
   *   Whether or not to support capture.
   */
  public function getRefund() {
    return $this->pluginDefinition['refund'];
  }

  /**
   * Get the Mollie API Client to use.
   */
  public function getMollieClient() {
    $profile_id = $this->pluginDefinition['profile'];
    $profile = MollieProfile::load($profile_id);

    return $profile->getMollieClient();
  }

  /**
   * {@inheritdoc}
   */
  protected function doExecutePayment() {
    // Get the Mollie client.
    $mollie_client = $this->getMollieClient();

    $redirect_url = new Url('mollie_payment.redirect',
      ['payment' => $this->getPayment()->id()], ['absolute' => TRUE]);
    $webhook_url = new Url('mollie_payment.webhook',
      ['payment' => $this->getPayment()->id()], ['absolute' => TRUE]);

    $payment_data = [
      'amount' => $this->getPayment()->getAmount(),
      'description' => $this->getPayment()->getPaymentType()->getPaymentDescription(),
      'redirectUrl' => $redirect_url->toString(),
      'webhookUrl' => $webhook_url->toString(),
      'metadata' => Json::encode(['id' => $this->getPayment()->id()]),
    ];

    try {
      $mollie_payment = $mollie_client->payments->create($payment_data);

      // Store the Mollie payment id.
      $this->getPayment()->set('mollie_payment_id', $mollie_payment->id)->save();

      // Redirect visitor to Mollie.
      $redirect_url = $mollie_payment->getPaymentUrl();

      $url = Url::fromUri($redirect_url);
      $response = new Response($url);
      return new OperationResult($response);
    }
    catch (Mollie_API_Exception $e) {
      drupal_set_message($e->getMessage(), 'error');
      return;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getPaymentExecutionResult() {
    // Get the Mollie client.
    $mollie_client = $this->getMollieClient();

    // Load the Mollie transaction.
    $mollie_transaction_id = $this->getPayment()->get('mollie_payment_id')->value;
    $mollie_payment = $mollie_client->payments->get($mollie_transaction_id);

    // Redirect visitor to Mollie.
    $response = new Response(Url::fromUri($mollie_payment->getPaymentUrl()));
    return new OperationResult($response);
  }

  /**
   * {@inheritdoc}
   */
  public function doCapturePayment() {
    $this->getPayment()->setPaymentStatus($this->paymentStatusManager->createInstance($this->getCaptureStatusId()));
    $this->getPayment()->save();
  }

  /**
   * {@inheritdoc}
   */
  public function doCapturePaymentAccess(AccountInterface $account) {
    return $this->getCapture() && $this->getPayment()->getPaymentStatus()->getPluginId() != $this->getCaptureStatusId();
  }

  /**
   * {@inheritdoc}
   */
  public function doRefundPayment() {
    $this->getPayment()->setPaymentStatus($this->paymentStatusManager->createInstance($this->getRefundStatusId()));
    $this->getPayment()->save();
  }

  /**
   * {@inheritdoc}
   */
  public function doRefundPaymentAccess(AccountInterface $account) {
    return $this->getRefund() && $this->getPayment()->getPaymentStatus()->getPluginId() != $this->getRefundStatusId();
  }

  /**
   * {@inheritdoc}
   */
  public function updatePaymentStatusAccess(AccountInterface $account) {
    return AccessResult::forbidden();
  }

  /**
   * {@inheritdoc}
   */
  public function getSettablePaymentStatuses(AccountInterface $account, PaymentInterface $payment) {
    return [];
  }

  /**
   * Update payment status,
   */
  public function updatePaymentStatus(Payment $payment, $payment_status) {
    $payment->setPaymentStatus($payment->getPaymentMethod()->paymentStatusManager->createInstance($payment_status));
    $payment->save();
  }

}
